'************************************************************************************* 
' 
' Peter Mather
' 
' 1.2 
' 2015-19-1 
' 
' Graphics library with drivers for SSD1289 display with XPT2046 touch controller
'
' drawing primitives assume a graphics display where top left is point 0, 0
'
' set screen orientation: sub T32.setrot(x) ;T32_normal=normal, T32_inverse=invert, T32_rt_right=rotate right, T32_rt_left=rotate left
' draw circle: sub T32.dcirc(x0, y0, r, colour) ;x, y coordinates of centre, radius, colour
' draw filled circle: sub T32.fcirc(x0, y0, r, colour) ;x, y coordinates of centre, radius, colour
' draw triangle:  sub T32.dtri(x0, y0, x1, y1, 	x2, y2, colour) ;three coordinate pairs, colour
' draw filled triangle sub T32.ftri (x0, y0, x1, y1, x2, y2, colour) ;three coordinate pairs, colour
' print string: sub T32.pstring(xx0, yy0, text$, sz, orientation, forecolour, backcolour) ;x, y coordinates of top left of first character, text string, size, character orientation,  foreground colour,  background colour
' print 8x5 bitmap: sub T32.pbitmap(xx0, yy0, char, sz, orientation, forecolour, backcolour) ;x, y coordinates of top left of first character, bitmap, size, character orientation,  foreground colour,  background colour
' draw line: sub T32.dline(xx0, yy0, xx1, yy1, colour) ;x, y of start line, x, y of end line, colour
' draw rectangle: sub T32.drect(x, y, w, h, colour) ;x, y coordinate of top left, width, height, colour
' draw filled rectangle: sub T32.frect(x, y, w, h, colour) ;x, y coordinate of top left, width, height, colour
' draw rounded rectangle: sub T32.drndrect(x, y, w, h, r, colour) ;x, y coordinates of top left, width, height, radius of corner, colour
' draw filled rounded rectangle: sub T32.frndrect(x, y, w, h, r, colour) ;x, y coordinates of top left, width, height, radius of corner, colour
'
option explicit
option default NONE
cpu 48
  T32.initdisplay

Do
   T32_test
loop

end
'
' 
 specific routines
'
sub T32.initdisplay
'Global "Constants"
  dim integer T32_CS=4 'b0
  dim integer T32_CD=5 'b1
  dim integer T32_WR=6 'b2
  dim integer LT_latch=7 'b3 connect to pin 12 on both 74HC595
  DIM integer Master_reset=15 ' connect to reset pins on display and both 74HC595 (pin 10)
' SPI Clock - connect to pin 11 on both 74HC595
' SPI Out - connect to pin 14 on first 74HC595
' connect pin 14 on second 74HC595 to pin 9 on first
' connect pin 13 on both 74HC595 to GND
' Tie display RD pin high
  dim integer BLACK = &H0000 ' Color definitions: 16 bit (RGB565) colour palette so these are just examples
  dim integer BLUE  = &H001F
  dim integer RED   = &HF800
  dim integer GREEN = &H07E0
  dim integer CYAN  = &H07FF
  dim integer MAGENTA=&HF81F
  dim integer YELLOW= &HFFE0 
  dim integer WHITE = &HFFFF
  dim integer T32.pix=0
  dim integer T32.ffr=1
  dim integer T32.filltri=2
  dim integer T32.drawline=3
  dim integer T32.writecommand=4
  dim integer T32.writedata=5
  dim integer T32.fpstring=7
  dim integer T32.fcirch=8
  dim integer T32.dcirch=9
  dim integer T32_normal=0
  dim integer T32_inverse=1
  dim integer T32_rt_right=2
  dim integer T32_rt_left=3
  ' Global variables
  dim integer T32_AddrOfFont,   T32_params(6) ' Global variables
  DIM integer T32_wi=0,T32_ht=1,T32_rot=2,T32_nr=3,T32_X=4,T32_Y=5,T32_seed=6
  T32_AddrOfFont=PEEK(CFunAddr FONT)  
  local integer i,j,k
  T32_params(T32_wi)=240
  T32_params(T32_ht)=320
  pin(LT_latch)=0
  setpin LT_latch,dout
  pin(T32_CS)=1
  setpin T32_CS, DOUT
  pin(T32_CD)=1
  setpin T32_CD, DOUT
  pin(T32_WR)=1
  setpin T32_WR, DOUT
  pin(Master_RESET)=1
  setpin Master_RESET, DOUT'  Resets the display and the 74HC595s
  pin(Master_RESET)=0
  pause 100
  pin(Master_RESET)=1
  pause 100
  spi open 1000000,3,8
  dim integer T_IRQ=10
  dim integer T_CS=9 
  setpin T_IRQ,din,pullup
  pin(T_CS)=1
  setpin T_CS,dOUT
  i=T.Getdata(&HA0) 'wake up the controller
 	T32.wreg(&H00,&H0001)
	T32.wreg(&H03,&HA8A4)
	T32.wreg(&H0C,&H0000)
	T32.wreg(&H0D,&H080C)
	T32.wreg(&H0E,&H2B00)
	T32.wreg(&H1E,&H00B7)
	T32.wreg(&H01,&H2B3F)
	T32.wreg(&H02,&H0600)
	T32.wreg(&H10,&H0000)
	T32.wreg(&H11,&H6070)
	T32.wreg(&H05,&H0000)
	T32.wreg(&H06,&H0000)
	T32.wreg(&H16,&HEF1C)
	T32.wreg(&H17,&H0003)
	T32.wreg(&H07,&H0233)
	T32.wreg(&H0B,&H0000)
	T32.wreg(&H0F,&H0000)
	T32.wreg(&H41,&H0000)
	T32.wreg(&H42,&H0000)
	T32.wreg(&H48,&H0000)
	T32.wreg(&H49,&H013F)
	T32.wreg(&H4A,&H0000)
	T32.wreg(&H4B,&H0000)
	T32.wreg(&H44,&HEF00)
	T32.wreg(&H45,&H0000)
	T32.wreg(&H46,&H013F)
	T32.wreg(&H30,&H0707)
	T32.wreg(&H31,&H0204)
	T32.wreg(&H32,&H0204)
	T32.wreg(&H33,&H0502)
	T32.wreg(&H34,&H0507)
	T32.wreg(&H35,&H0204)
	T32.wreg(&H36,&H0204)
	T32.wreg(&H37,&H0502)
	T32.wreg(&H3A,&H0302)
	T32.wreg(&H3B,&H0302)
	T32.wreg(&H23,&H0000)
	T32.wreg(&H24,&H0000)
	T32.wreg(&H25,&H8000)
	T32.wreg(&H4f,&H0000)
	T32.wreg(&H4e,&H0000)
  pause 50
  i=T32draw(T32.writecommand, &H22)
  T32.setrot(T32_normal)
end sub
'
sub T32.wreg(register as integer, databyte as integer)
  local integer i
  i=T32draw(T32.writecommand, register)
  i=T32draw(T32.writedata, databyte)
end sub
'
sub T32.fillscreen(scol as integer)
  local integer i=T32_params(T32_rot)
  T32_params(T32_rot)=0
  local integer j=T32draw(T32.ffr,scol, 0,0,T32_params(T32_wi),T32_params(T32_ht), T32_params())
  T32_params(T32_rot)=i
end sub
'
function T.Getdata(command as integer) as integer 'get data from the touch controller
  local integer i,j,k
  poke word &HBF805800,&H8060 '8-bit
  poke word &HBF805830,&H17 'slow down the SPI
  pin(T_CS)=0
  i=spi(command)
  j=spi(0)
  k=spi(0)
  pin(T_CS)=1
  T.Getdata= (j*256+k)>>4
  poke word &HBF805830,1 'SPI back to full speed
  poke word &HBF805800,&H8460 '16-bit
end function
'
function T.getxy(x as integer, y as integer, timeout as integer) as integer
  timer=0
  do while timer<timeout and pin(T_IRQ)=1:loop
  if timer>=timeout then 
    T.getxy=0
  else
    y=T.getdata(&H90)
    x=T.getdata(&HD0)
    T.getxy=1
  endif
end function
'
'
' Generic drawing routines
'
sub T32.dcirc(x0 as integer, y0 as integer, r as integer, colour as integer) 'x, y coordinates of centre, radius
  local integer i=T32draw(T32.dcirch, colour,x0,y0,r,31,T32_params())

end sub
 '
sub T32.dtri(x0 as integer, y0 as integer, x1 as integer, y1 as integer, x2 as integer, y2 as integer, colour as integer) 'three coordinate pairs
  T32.dline(x0, y0, x1, y1, colour)
  T32.dline(x1, y1, x2, y2, colour)
  T32.dline(x2, y2, x0, y0, colour)
end sub
 '
sub T32.ftri(x0 as integer, y0 as integer, x1 as integer, y1 as integer, x2 as integer, y2 as integer, colour as integer) 'three coordinate pairs
  local integer i=T32draw(T32.filltri, x0,y0,x1,y1,x2,y2,colour,T32_params())
end sub
'
sub T32.pstring(xx0 as integer, yy0 as integer, text$, sz as integer, orientation as integer, fontcolour as integer, backgroundcolour as integer) 'x, y coordinates of top left of first character, text string, size, orientation
  local integer i=T32draw(T32.fpstring,T32_AddrOfFont,xx0,yy0,sz,orientation,fontcolour,backgroundcolour,T32_params(),text$)
end sub
'
sub T32.pbitmap(xx0 as integer, yy0 as integer, char as integer, sz as integer, orientation as integer, fontcolour as integer, backgroundcolour as integer) 'x, y coordinates of top left of first character, text string, size, orientation
  local integer i=T32draw(T32.fpstring,char,xx0,yy0,sz,orientation,fontcolour,backgroundcolour,T32_params(),"")
end sub
'
sub T32.dline(xx0 as integer, yy0 as integer, xx1 as integer, yy1 as integer, colour as integer) ' x, y of start line, x, y of end line
   local integer dp=T32draw(T32.drawline, colour, xx0, yy0, xx1, yy1,  T32_params())
end sub
 '
sub swap (a% , b% )
  local  t%
  t%=b%
  b%=a%
  a%=t%
end sub
 '
sub T32.drect(x as integer, y as integer, w as integer, h as integer, rcolour as integer) 'x, y coordinate of top left, wi, ht
  local integer a, dp
  dp=T32draw(T32.ffr, rcolour, x, y, w, 1,  T32_params()) 
  a=y+h-1
  dp=T32draw(T32.ffr, rcolour, x, a, w, 1,  T32_params()) 
  dp=T32draw(T32.ffr, rcolour, x, y, 1, h,  T32_params()) 
  a=x+w-1
  dp=T32draw(T32.ffr, rcolour, a, y, 1, h,  T32_params()) 
end sub
'
sub T32.frect(x as integer, y as integer, w as integer, h as integer, rcolour as integer) 'x, y coordinate of top left, width, height
  local integer dp
  dp=T32draw(T32.ffr, rcolour, x, y, w, h,  T32_params()) 
end sub
'
FUNCTION T32.Getchar(char as integer) as integer
  local integer i=char<<3
  T32.Getchar=PEEK(WORD T32_AddrOfFont+i)+(PEEK(WORD T32_AddrOfFont+i+4)<<32) 
END FUNCTION
'
sub T32.fcirc(x0 as integer, y0 as integer, r as integer, ccolour as integer)  'x, y coordinates of centre, radius
  local integer dp
  dp=T32draw(T32.ffr, ccolour, x0, y0-r, 1, 2*r+1,  T32_params())
  dp=T32draw(T32.fcircH,x0, y0, r, 3, 0, ccolour, T32_params())
end sub
 '
sub T32.drndrect(x as integer, y as integer, w as integer, h as integer, r as integer, rcolour as integer) 'x, y coordinates of top left, width, height, radius of corner
  local integer dp
  dp=T32draw(T32.ffr, rcolour, x+r, y, w-2*r, 1,  T32_params()) 
  dp=T32draw(T32.ffr, rcolour, x+r, y+h-1, w-2*r, 1,  T32_params()) 
  dp=T32draw(T32.ffr, rcolour, x, y+r, 1, h-2*r,  T32_params()) 
  dp=T32draw(T32.ffr, rcolour, x+w-1, y+r, 1, h-2*r,  T32_params()) 
  dp=T32draw(T32.dcirch, rcolour, x+r, y+r, r, 1,T32_params())
  dp=T32draw(T32.dcirch,  rcolour, x+w-r-1, y+r, r, 2,T32_params())
  dp=T32draw(T32.dcirch,  rcolour, x+w-r-1, y+h-r-1, r, 4,T32_params())
  dp=T32draw(T32.dcirch,  rcolour, x+r, y+h-r-1, r, 8,T32_params())
end sub
'
sub T32.frndrect(x as integer, y as integer, w as integer, h as integer, r as integer, colour as integer) 'x, y coordinates of top left, width, height, radius of corner
  local integer dp
  dp=T32draw(T32.ffr, colour, x+r, y, w-2*r, h,  T32_params()) 
  dp=T32draw(T32.fcircH,x+w-r-1, y+r, r, 1, h-2*r-1, colour, T32_params())
  dp=T32draw(T32.fcircH,x+r, y+r, r, 2, h-2*r-1, colour, T32_params())
end sub
 '
sub T32.setrot(x%) 'sets screen rotation: 0=T32_normal, 1=invert, 2=rotate right, 3-rotate left
   T32_params(T32_rot)=(x% and 3)
end sub
'
'D:\Peter\MPLABXProjects\TFT32\dist\default\production\T32draw.bas

'
'T32draw  2015-01-19 21:37:14 CFuncGen Ver 1.0.21 by user=Peter
'
CFunction T32draw
'writeCommand
	00000711
	3c02bf88 24030001 ac436134 24030002 ac436134 24030004 ac436138 ac436134 
	3c02bf80 ac445820 3c03bf80 8c625810 30420080 1040fffd 3c02bf80 8c425820 
	3c02bf88 24030008 ac436138 ac436134 24030004 ac436138 24030001 ac436138 
	03e00008 00000000 

'writeData
3c02bf88 24030001 ac436134 24030002 ac436138 24030004 
	ac436138 ac436134 3c02bf80 ac445820 3c03bf80 8c625810 30420080 1040fffd 
	3c02bf80 8c425820 3c02bf88 24030008 ac436138 ac436134 24030004 ac436138 
	24030001 ac436138 03e00008 00000000 

'localfastfillrect
27bdffc8 afbf0034 afb70030 afb6002c 
	afb50028 afb40024 afb30020 afb2001c afb10018 afb00014 00808021 8faa0048 
	8fa9004c 8fac0050 8fad0054 8fae0058 8faf005c 8fa20060 8c4b0000 8c480010 
	1500002d 8c580008 04e00005 00002821 14a70103 00cb102b 1040010a 00002021 
	24c2ffff 0046202b 24e3ffff 00832021 004c1821 0062102b 008d2021 00441021 
	04400006 00609021 54a20004 2572ffff 006b182b 50600001 2572ffff 05200005 
	00002021 148900f1 0158102b 104000f5 00002821 2542ffff 004a282b 2523ffff 
	00a32821 004e1821 0062102b 00af2821 00451021 044000f8 00609821 548200f6 
	2712ffff 0078182b 546000f4 0140a021 100000f1 2712ffff 24020001 5502002c 
	24020002 00063027 00073827 00cb5821 0166302b 00c73021 01609021 00002021 
	04c000d8 00002821 000a5027 00094827 0158c021 030a502b 01494821 052000d1 
	03009821 24020001 004c6023 004c102b 00c23023 018b5821 016c602b 00cd6823 
	018dc821 00191fc3 00032827 000517c3 01622024 00808821 24020001 004e7023 
	004e102b 01224823 01d8c021 030e702b 012f7823 01cf1021 00021fc3 00032827 
	000517c3 03022024 10000067 0080a021 15020036 000a2027 00091827 008b1021 
	0044202b 00831821 00409021 04600008 00602021 54600005 2572ffff 004b102b 
	14400004 24020001 2572ffff 00002021 24020001 004e2823 0045102b 00822023 
	00b21821 0065282b 008f2023 00608821 00a4c821 00192fc3 00051827 000327c3 
	02241024 00408821 04e00005 00002021 14870090 00d8102b 10400092 00002821 
	24c2ffff 0046282b 24e3ffff 00a32821 004c1821 0062102b 00ad2821 00451021 
	04400009 00609821 54820005 2713ffff 0078182b 54600005 00c0a021 2713ffff 
	10000002 00c0a021 00c0a021 24020003 1502002e 26570001 05200005 00001821 
	14690076 014b102b 10400075 00002021 2542ffff 004a282b 2524ffff 00a42021 
	004e7021 01c2102b 008f7821 004f1021 04400007 01c09021 54620005 2572ffff 
	01cb702b 55c00003 00063027 2572ffff 00063027 00073827 00d8c021 0306302b 
	00c73021 03009821 00002021 04c0005d 00002821 24020001 004c6023 004c102b 
	00c23023 0198c021 030c602b 00cd6823 018d1021 0300a021 04410003 01408821 
	01408821 0000a021 26570001 02f1b823 26620001 00541023 72e2b802 24040044 
	0411FEEF 00000000 00122200 00912021 0411FF05 00000000 24040045 0411FEE8 
	00000000 02802021 0411FEFF 00000000 24040046 0411FEE2 00000000 02602021 
	0411FEF9 00000000 2404004e 0411FEDC 00000000 02202021 0411FEF3 00000000 
	2404004f 0411FED6 00000000 02802021 0411FEED 00000000 24040022 0411FED0 
	00000000 3c02bf88 24030001 ac436134 24030002 ac436138 24030004 ac436138 
	12e00011 26f7ffff 3c04bf88 24060004 3c03bf80 24050008 ac866134 ac705820 
	8c625810 30420080 1040fffd 00000000 8c625820 ac856138 ac856134 ac866138 
	16e0fff5 26f7ffff 24030001 3c02bf88 ac436138 24040001 1000000a 00002821 
	00002021 10000007 00002821 10000005 00002821 10000003 00002821 00002021 
	00002821 00801021 00a01821 8fbf0034 8fb70030 8fb6002c 8fb50028 8fb40024 
	8fb30020 8fb2001c 8fb10018 8fb00014 03e00008 27bd0038 0140a021 1000ff73 
	00c08821 

'localdrawPixel
27bdffe0 afbf001c afb20018 afb10014 afb00010 00808021 8fa30030 
	8fa20038 8c450000 8c440008 8c420010 24070001 14470006 00608821 00069027 
	02459021 00038827 1000000e 02248821 24070002 14470005 00c09021 00039027 
	00c08821 10000007 02459021 24070003 14470005 02511027 00068827 00609021 
	02248821 02511027 000217c2 0245282a 0224202a 00a42024 00441024 00002021 
	10400015 00002821 2404004e 0411FE6C 00000000 02402021 0411FE83 00000000 
	2404004f 0411FE66 00000000 02202021 0411FE7D 00000000 24040022 0411FE60 
	00000000 02002021 0411FE77 00000000 24040001 00002821 00801021 00a01821 
	8fbf001c 8fb20018 8fb10014 8fb00010 03e00008 27bd0020 

'swap
8c820000 8ca30000 
	ac830000 03e00008 aca20000 

'drawline
27bdffa8 afbf0054 afb70050 afb6004c afb50048 
	afb40044 afb30040 afb2003c afb10038 afb00034 0080a021 00a0a821 8fa40068 
	8fa20070 8fa30078 8fb30080 afa60020 afa40024 afa20028 afa3002c 0083882a 
	00642823 00831823 0071280a 00c2202a 00461823 00c21023 0064100b 0045882a 
	1220000a 8fa20028 27a40020 27a50024 0411FFD9 00000000 27a40028 27a5002c 
	0411FFD5 00000000 8fa20028 8fa30020 0043102a 1040000a 8fa30028 27a40020 
	27a50028 0411FFCC 00000000 27a40024 27a5002c 0411FFC8 00000000 8fa30028 
	8fa20020 8fa4002c 8fb20024 0244282a 10a00007 0062b023 00929023 001687c2 
	02168021 00108043 10000006 24170001 02449023 001687c2 02168021 00108043 
	2417ffff 0062182a 14600028 8fbf0054 1220000e 8fa30024 8fa70024 afa20010 
	000217c3 afa20014 afb30018 02802021 02a02821 00e03021 00073fc3 0411FF61 
	00000000 1000000c 02128023 afa30010 00031fc3 afa30014 afb30018 02802021 
	02a02821 00403021 00023fc3 0411FF55 00000000 02128023 06010006 8fa20020 
	8fa20024 00571021 afa20024 02168021 8fa20020 24420001 afa20020 8fa30028 
	0062182a 1060ffda 8fbf0054 8fb70050 8fb6004c 8fb50048 8fb40044 8fb30040 
	8fb2003c 8fb10038 8fb00034 03e00008 27bd0058 

'drawchar
27bdff68 afbf0094 afbe0090 
	afb7008c afb60088 afb50084 afb40080 afb3007c afb20078 afb10074 afb00070 
	afa600a0 afa700a4 8fa200b0 afa2005c 8fa300b8 afa30060 8fa200c0 afa20058 
	8fb400d0 8fa3005c afa30048 8fa20060 afa2004c 8fb00058 8fa300c8 afa30050 
	afa40030 14600061 afa50034 00102823 afa50054 00101080 00501021 8fa5005c 
	00a21021 afa20044 24a20005 afa20040 27b60030 27a30036 afa30038 00009021 
	24170001 241e0008 02008821 10000049 0280a821 92c20000 02021007 30420001 
	1040001c 00000000 5637000c afb30010 afb40010 afb20014 afb50018 8fa400a0 
	8fa500a4 8fa60040 02403821 0411FEFD 00000000 10000029 26100001 afb20014 
	afb10018 afb2001c afb10020 afb20024 afb50028 8fa400a0 8fa500a4 8fa6003c 
	02403821 0411FDBA 00000000 1000001b 26100001 5637000c afb30010 afb40010 
	afb20014 afb50018 8fa400a8 8fa500ac 8fa60040 02403821 0411FEE2 00000000 
	1000000e 26100001 afb20014 afb10018 afb2001c afb10020 afb20024 afb50028 
	8fa400a8 8fa500ac 8fa6003c 02403821 0411FD9F 00000000 26100001 02719821 
	161effc4 26940001 8fa50044 8fa20054 00a22821 afa50044 8fa30040 2463ffff 
	afa30040 26d60001 8fa50038 52c50078 02208021 8fa20044 afa2003c 8fb4004c 
	02809821 1000ffb3 02408021 24020001 8fa30050 14620071 24020002 8fa50058 
	00052823 afa50048 8fa3005c 24620001 8fa50058 00451023 afa20044 2462fffa 
	afa20054 afa30040 00009021 8fa5004c 00b01823 24630001 001010c0 00501023 
	00621023 afa20064 27a20030 8fa3005c 00431023 afa20068 02008821 10000048 
	0280f021 92a20005 02021007 30420001 1040001c 00000000 5637000c afb30010 
	afb40010 afb20014 afbe0018 8fa400a0 8fa500a4 8fa60040 02403821 0411FE91 
	00000000 10000029 2610ffff afb20014 afb10018 afb2001c afb10020 afb20024 
	afbe0028 8fa400a0 8fa500a4 8fa6003c 02403821 0411FD4E 00000000 1000001b 
	2610ffff 5637000c afb30010 afb40010 afb20014 afbe0018 8fa400a8 8fa500ac 
	8fa60040 02403821 0411FE76 00000000 1000000e 2610ffff afb20014 afb10018 
	afb2001c afb10020 afb20024 afbe0028 8fa400a8 8fa500ac 8fa6003c 02403821 
	0411FD33 00000000 2610ffff 02719821 1616ffc4 26940001 8fa50044 8fa20048 
	00a22821 afa50044 8fa30040 2463ffff afa30040 8fa50054 50650073 02208021 
	8fa20044 afa2003c 8fb30064 8fa3004c 2474fff9 24100007 8fa50068 8fa20040 
	00a2a821 24170001 1000ffae 2416ffff 02a0a021 24020002 8fa30050 14620064 
	24020003 8fa50058 00052823 afa50050 8fa3005c 24620001 8fa50058 00451023 
	afa20044 afa00040 24160001 00009021 27be002f 02008821 1000004b 0280b821 
	92020000 02a21024 1040001c 00000000 5636000c afb30010 afb40010 afb20014 
	afb70018 8fa400a0 8fa500a4 8fa60048 02403821 0411FE2B 00000000 10000029 
	2610ffff afb20014 afb10018 afb2001c afb10020 afb20024 afb70028 8fa400a0 
	8fa500a4 8fa6003c 02403821 0411FCE8 00000000 1000001b 2610ffff 5636000c 
	afb30010 afb40010 afb20014 afb70018 8fa400a8 8fa500ac 8fa60048 02403821 
	0411FE10 00000000 1000000e 2610ffff afb20014 afb10018 afb2001c afb10020 
	afb20024 afb70028 8fa400a8 8fa500ac 8fa6003c 02403821 0411FCCD 00000000 
	2610ffff 02719821 161effc5 26940001 8fa20040 24420001 afa20040 8fa30044 
	8fa50050 00651821 afa30044 8fa20048 2442ffff afa20048 8fa30040 24050008 
	1065007c 8fbf0094 8fa20040 0056a804 8fa30044 afa3003c 27b00035 8fb4004c 
	1000ffaf 02809821 03c0a021 24020003 8fa50050 14a2006e 8fa50058 8fa20060 
	24440001 00051840 000510c0 00431823 00831823 afa30050 00051823 afa30054 
	00451023 8fa5005c 00a21021 afa20048 24a20007 afa20044 24030007 afa30040 
	24170001 00009021 27a50036 afa50038 1000004d 0280f021 92220000 02c21024 
	1040001c 00000000 5617000c afb40010 afb50010 afb20014 afbe0018 8fa400a0 
	8fa500a4 8fa60044 02403821 0411FDBD 00000000 10000029 26310001 afb20014 
	afb00018 afb2001c afb00020 afb20024 afbe0028 8fa400a0 8fa500a4 8fa6003c 
	02403821 0411FC7A 00000000 1000001b 26310001 5617000c afb30010 afb50010 
	afb20014 afbe0018 8fa400a8 8fa500ac 8fa60044 02403821 0411FDA2 00000000 
	1000000e 26310001 afb20014 afb00018 afb2001c afb00020 afb20024 afbe0028 
	8fa400a8 8fa500ac 8fa6003c 02403821 0411FC5F 00000000 26310001 0290a021 
	26b50001 8fa20038 1622ffc3 02709821 8fa30040 2463ffff afa30040 8fa50048 
	8fa20054 00a22821 afa50048 8fa30044 2463ffff afa30044 8fa50040 2402ffff 
	10a2000c 8fbf0094 8fa30040 0077b004 8fa50048 afa5003c 27b10030 8fa2004c 
	2455fffb 8fb30050 1000ffab 0260a021 8fbf0094 8fbe0090 8fb7008c 8fb60088 
	8fb50084 8fb40080 8fb3007c 8fb20078 8fb10074 8fb00070 03e00008 27bd0098 

'nextchar
	14e00008 24020001 00061040 000618c0 00621023 8c830000 00621021 10000012 
	ac820000 14e20007 00061040 000630c0 00463023 8c820000 00463021 03e00008 
	ac860000 24020002 14e20007 00061040 000630c0 00c23023 8ca20000 00463021 
	03e00008 aca60000 24020003 14e20006 00061040 000630c0 00463023 8ca20000 
	00463021 aca60000 03e00008 00000000 

'pstring
27bdff78 afbf0084 afbe0080 afb7007c 
	afb60078 afb50074 afb40070 afb3006c afb20068 afb10064 afb00060 8fa30098 
	8fb000a0 8fa200a8 afa20048 8fa700b0 8fa800b8 8fb500c0 8fb400c4 afa60040 
	afa30044 02009021 00409821 00e0b021 afa40058 92820000 1040003f 0100b821 
	1840005a 24100001 00071fc3 afa30048 000817c3 afa2004c 00121fc3 afa30050 
	001317c3 afa20054 02901021 90420000 2443ffe0 306300ff 2c630060 50600008 
	00002021 2442ffe0 000210c0 8fa30058 00431021 8c440000 10000002 8c450004 
	00002821 afb70010 8fa2004c afa20014 8fa20040 afa20018 000217c3 afa2001c 
	8fa20044 afa20020 000217c3 afa20024 afb20028 8fa30050 afa3002c afb30030 
	8fa20054 afa20034 afb50038 02c03021 8fa70048 0411FDBF 00000000 00408821 
	0060f021 27a40040 27a50044 02403021 02603821 0411FF8A 00000000 26100001 
	92820000 0050102a 5040ffd0 02901021 1000001f 8fa20040 afa80010 000847c3 
	afa80014 afa60018 000637c3 afa6001c afa30020 00031fc3 afa30024 afb00028 
	001017c3 afa2002c 8fa30048 afa30030 000317c3 afa20034 afb50038 00e03021 
	00073fc3 0411FD9B 00000000 00408821 0060f021 27a40040 27a50044 02003021 
	8fa70048 0411FF66 00000000 8fa20040 aea20020 000217c3 aea20024 8fa20044 
	aea20028 000217c3 aea2002c 02201021 03c01821 8fbf0084 8fbe0080 8fb7007c 
	8fb60078 8fb50074 8fb40070 8fb3006c 8fb20068 8fb10064 8fb00060 03e00008 
	27bd0088 

'dcirch
27bdffa0 afbf005c afbe0058 afb70054 afb60050 afb5004c afb40048 
	afb30044 afb20040 afb1003c afb00038 afa40060 afa50064 00c0b021 8fb50070 
	8fb00078 8fa20080 8fb70088 afa60024 afb50028 afa20020 30420010 1040002a 
	02008821 000697c3 02151021 afa20010 000217c3 afa20014 afb70018 02403821 
	0411FC98 00000000 02b01023 afa20010 000217c3 afa20014 afb70018 8fa40060 
	8fa50064 02c03021 02403821 0411FC8D 00000000 001597c3 02163821 afb50010 
	afb20014 afb70018 8fa40060 8fa50064 00e03021 00073fc3 0411FC82 00000000 
	02d03823 afb50010 afb20014 afb70018 8fa40060 8fa50064 00e03021 00073fc3 
	0411FC78 00000000 1a20008f 24110001 02308823 0010f023 001ef040 26b3ffff 
	26d4ffff 26b50001 26d60001 00009021 8fa20020 30420004 afa2002c 8fa30020 
	30630002 afa30030 8fa20020 30420008 afa20034 8fa30020 30630001 afa30020 
	06200004 8fa2002c 2610ffff 27de0002 023e8821 10400019 26520001 8fa30028 
	02031021 afa20010 000217c3 afa20014 afb70018 8fa40060 8fa50064 02c03021 
	00163fc3 0411FC4F 00000000 8fa20024 02023821 afb50010 001517c3 afa20014 
	afb70018 8fa40060 8fa50064 00e03021 00073fc3 0411FC43 00000000 8fa30030 
	1060001a 8fa30034 8fa30028 00701023 afa20010 000217c3 afa20014 afb70018 
	8fa40060 8fa50064 02c03021 00163fc3 0411FC34 00000000 8fa20024 02023821 
	afb30010 001317c3 afa20014 afb70018 8fa40060 8fa50064 00e03021 00073fc3 
	0411FC28 00000000 8fa30034 1060001a 8fa20020 8fa20024 00503823 afb50010 
	001517c3 afa20014 afb70018 8fa40060 8fa50064 00e03021 00073fc3 0411FC19 
	00000000 8fa30028 02031021 afa20010 000217c3 afa20014 afb70018 8fa40060 
	8fa50064 02803021 00143fc3 0411FC0D 00000000 8fa20020 5040001a 2673ffff 
	8fa30024 00703823 afb30010 001317c3 afa20014 afb70018 8fa40060 8fa50064 
	00e03021 00073fc3 0411FBFE 00000000 8fa30028 00701023 afa20010 000217c3 
	afa20014 afb70018 8fa40060 8fa50064 02803021 00143fc3 0411FBF2 00000000 
	2673ffff 2694ffff 26b50001 0250102a 10400005 26d60001 00121040 24420001 
	1000ff87 02228821 8fbf005c 8fbe0058 8fb70054 8fb60050 8fb5004c 8fb40048 
	8fb30044 8fb20040 8fb1003c 8fb00038 03e00008 27bd0060 

'fcirch
27bdff90 afbf006c 
	afbe0068 afb70064 afb60060 afb5005c afb40058 afb30054 afb20050 afb1004c 
	afb00048 8fb10080 8fb40090 8fbe00a0 afa40038 afa6003c 8fa20088 1a200076 
	02801821 24100001 02118023 0011b823 0017b840 2495ffff 24d3ffff 26940003 
	24960001 00009021 30440001 afa40030 24630001 afa30040 30420002 afa20034 
	06000004 8fa20030 2631ffff 26f70002 02178021 1040002a 26520001 8fa3003c 
	00711023 afa20010 000217c3 afa20014 24020001 00001821 afa20018 afa3001c 
	00111040 8fa30040 00621021 afa20020 000217c3 afa20024 afbe0028 8fa40098 
	8fa5009c 02c03021 00163fc3 0411FA68 00000000 8fa40038 02243821 afb30010 
	001317c3 afa20014 24020001 00001821 afa20018 afa3001c afb40020 001417c3 
	afa20024 afbe0028 8fa40098 8fa5009c 00e03021 00073fc3 0411FA55 00000000 
	8fa30034 5060002b 26b5ffff 8fa4003c 00911023 afa20010 000217c3 afa20014 
	24020001 00001821 afa20018 afa3001c 00111040 8fa30040 00621021 afa20020 
	000217c3 afa20024 afbe0028 8fa40098 8fa5009c 02a03021 00153fc3 0411FA3C 
	00000000 8fa40038 00913823 afb30010 001317c3 afa20014 24020001 00001821 
	afa20018 afa3001c afb40020 001417c3 afa20024 afbe0028 8fa40098 8fa5009c 
	00e03021 00073fc3 0411FA29 00000000 26b5ffff 2673ffff 26940002 0251102a 
	10400005 26d60001 00121040 24420001 1000ff9b 02028021 8fbf006c 8fbe0068 
	8fb70064 8fb60060 8fb5005c 8fb40058 8fb30054 8fb20050 8fb1004c 8fb00048 
	03e00008 27bd0070 

'ftri
27bdff88 afbf0074 afb70070 afb6006c afb50068 afb40064 
	afb30060 afb2005c afb10058 afb00054 8fa20090 8fb400a8 8fb500ac 8fb100b0 
	afa40038 afa6003c 8fa30088 afa30040 afa20044 8fa30098 afa30048 8fa300a0 
	0046102a 10400009 afa3004c 27a4003c 27a50044 0411FB70 00000000 27a40038 
	27a50040 0411FB6C 00000000 8fa2004c 8fa30044 0043102a 1040000a 8fa20044 
	27a4004c 27a50044 0411FB63 00000000 27a40048 27a50040 0411FB5F 00000000 
	8fa20044 8fa3003c 0043102a 1040000a 8fb0003c 27a4003c 27a50044 0411FB56 
	00000000 27a40038 27a50040 0411FB52 00000000 8fb0003c 8fa2004c 1602002e 
	8fb20044 8fa20038 afa20030 afa20034 8fa30040 0062202a 10800003 0043102a 
	10000003 afa30030 54400001 afa30034 8fa20048 8fa30030 0043182a 10600003 
	8fa30034 10000004 afa20030 0062182a 54600001 afa20034 8fa70030 8fa2003c 
	afa20010 000217c3 afa20014 24020001 00001821 afa20018 afa3001c 8fa20034 
	24420001 00471023 afa20020 000217c3 afa20024 afb10028 02802021 02a02821 
	00e03021 00073fc3 0411F9A9 00000000 1000007a 8fbf0074 

'.LBB2
00521026 0002102b 
	02429023 0250102a 14400039 8fa2004c 24160001 0000b821 8fa20038 8fa3003c 
	02032823 8fa60040 00c23023 70a63002 8fa40044 00832023 00c4001a 008001f4 
	00002012 00822021 afa40030 8fa60048 00c23023 70a62802 8fa6004c 00c31823 
	00a3001a 006001f4 00001812 00621021 afa20034 0044102a 10400006 8fa70030 
	27a40030 27a50034 0411FAFB 00000000 8fa70030 afb00010 001017c3 afa20014 
	8fa20034 24420001 00471023 afa20018 000217c3 afa2001c afb60020 afb70024 
	afb10028 02802021 02a02821 00e03021 00073fc3 0411F96E 00000000 26100001 
	0250102a 1040ffcd 8fa20038 8fa2004c 0050182a 14600038 24120001 00009821 
	8fa60040 8fa40038 8fa50048 8fa80044 02083823 00a61823 70e33802 00484023 
	00e8001a 010001f4 00001812 00661821 afa30030 8fa7003c 02073023 00a42823 
	70c53002 00471023 00c2001a 004001f4 00001012 00442021 afa40034 0083202a 
	10800006 8fa70030 27a40030 27a50034 0411FAC1 00000000 8fa70030 afb00010 
	001017c3 afa20014 8fa20034 24420001 00471023 afa20018 000217c3 afa2001c 
	afb20020 afb30024 afb10028 02802021 02a02821 00e03021 00073fc3 0411F934 
	00000000 26100001 8fa2004c 0050182a 1060ffcc 8fa60040 

'.LBE2
8fbf0074 8fb70070 
	8fb6006c 8fb50068 8fb40064 8fb30060 8fb2005c 8fb10058 8fb00054 03e00008 
	27bd0078 

'main
27bdffb8 afbf0044 afb00040 00808021 00e01021 8faa0058 8fa9005c 
	8fa80060 8fac0064 8c830000 8c840004 00643825 14e00010 8fab0068 8ca50000 
	8cc70000 8c420000 afa20010 000217c3 afa20014 afaa0018 00a02021 00052fc3 
	00e03021 00073fc3 0411FA3E 00000000 100000c6 8fbf0044 24070001 14670018 
	24070002 14800016 00000000 8ca40000 8ca50004 8cc70004 8cc60000 8c430004 
	8c420000 afa20010 afa30014 8d420000 8d430004 afa20018 afa3001c 8d220000 
	8d230004 afa20020 afa30024 afa80028 0411F8EF 00000000 100000ac 8fbf0044 
	14670021 24070003 1480001f 00000000 8ca40000 8ca50004 8cc70004 8cc60000 
	8c430004 8c420000 afa20010 afa30014 8d420000 8d430004 afa20018 afa3001c 
	8d220000 8d230004 afa20020 afa30024 8d020000 8d030004 afa20028 afa3002c 
	8d820000 8d830004 afa20030 afa30034 afab0038 0411FEBC 00000000 8e020000 
	10000089 8e030004 14670019 24070004 14800017 00000000 8ca40000 8ca50004 
	8cc70004 8cc60000 8c420000 afa20010 000217c3 afa20014 8d420000 afa20018 
	000217c3 afa2001c 8d220000 afa20020 000217c3 afa20024 afa80028 0411FA33 
	00000000 8e020000 1000006f 8e030004 14670009 24070005 14800007 00000000 
	8ca40000 0411F876 00000000 8e020000 10000065 8e030004 14670009 24070007 
	14800007 00000000 8ca40000 0411F886 00000000 8e020000 1000005b 8e030004 
	14670022 24070008 14800020 00000000 8ca40000 8ca50004 8cc70004 8cc60000 
	8c430004 8c420000 afa20010 afa30014 8d420000 8d430004 afa20018 afa3001c 
	8d220000 8d230004 afa20020 afa30024 8d020000 8d030004 afa20028 afa3002c 
	8d820000 8d830004 afa20030 afa30034 afab0038 8fa2006c afa2003c 0411FC6C 
	00000000 10000039 8fbf0044 1467001d 24070009 1480001b 00000000 8ca40000 
	8ca50004 8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d420000 
	8d430004 afa20018 afa3001c 8d220000 8d230004 afa20020 afa30024 8d020000 
	8d030004 afa20028 afa3002c afac0030 0411FDB9 00000000 8e020000 1000001a 
	8e030004 14670019 8fbf0044 54800018 8fb00040 8ca40000 8ca50004 8cc70004 
	8cc60000 8c430004 8c420000 afa20010 afa30014 8d420000 8d430004 afa20018 
	afa3001c 8d220000 8d230004 afa20020 afa30024 afa80028 0411FCC2 00000000 
	8e020000 8e030004 8fbf0044 8fb00040 03e00008 27bd0048 
End CFunction
'
'main
27bdffb8 afbf0044 afb00040 00808021 
	00e01021 8faa0058 8fa9005c 8fa80060 8fac0064 8c830000 8c840004 00643825 
	14e00010 8fab0068 8ca50000 8cc70000 8c420000 afa20010 000217c3 afa20014 
	afaa0018 00a02021 00052fc3 00e03021 00073fc3 0411FA3E 00000000 100000c6 
	8fbf0044 24070001 14670018 24070002 14800016 00000000 8ca40000 8ca50004 
	8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d420000 8d430004 
	afa20018 afa3001c 8d220000 8d230004 afa20020 afa30024 afa80028 0411F8E6 
	00000000 100000ac 8fbf0044 14670021 24070003 1480001f 00000000 8ca40000 
	8ca50004 8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d420000 
	8d430004 afa20018 afa3001c 8d220000 8d230004 afa20020 afa30024 8d020000 
	8d030004 afa20028 afa3002c 8d820000 8d830004 afa20030 afa30034 afab0038 
	0411FEBC 00000000 8e020000 10000089 8e030004 14670019 24070004 14800017 
	00000000 8ca40000 8ca50004 8cc70004 8cc60000 8c420000 afa20010 000217c3 
	afa20014 8d420000 afa20018 000217c3 afa2001c 8d220000 afa20020 000217c3 
	afa20024 afa80028 0411FA33 00000000 8e020000 1000006f 8e030004 14670009 
	24070005 14800007 00000000 8ca40000 0411F85B 00000000 8e020000 10000065 
	8e030004 14670009 24070007 14800007 00000000 8ca40000 0411F874 00000000 
	8e020000 1000005b 8e030004 14670022 24070008 14800020 00000000 8ca40000 
	8ca50004 8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 8d420000 
	8d430004 afa20018 afa3001c 8d220000 8d230004 afa20020 afa30024 8d020000 
	8d030004 afa20028 afa3002c 8d820000 8d830004 afa20030 afa30034 afab0038 
	8fa2006c afa2003c 0411FC6C 00000000 10000039 8fbf0044 1467001d 24070009 
	1480001b 00000000 8ca40000 8ca50004 8cc70004 8cc60000 8c430004 8c420000 
	afa20010 afa30014 8d420000 8d430004 afa20018 afa3001c 8d220000 8d230004 
	afa20020 afa30024 8d020000 8d030004 afa20028 afa3002c afac0030 0411FDB9 
	00000000 8e020000 1000001a 8e030004 14670019 8fbf0044 54800018 8fb00040 
	8ca40000 8ca50004 8cc70004 8cc60000 8c430004 8c420000 afa20010 afa30014 
	8d420000 8d430004 afa20018 afa3001c 8d220000 8d230004 afa20020 afa30024 
	afa80028 0411FCC2 00000000 8e020000 8e030004 8fbf0044 8fb00040 03e00008 
	27bd0048 
End CFunction
'
CFunction font
	00000000
	00000000 00000000 5f000000 00000000 00070000 00000007 147f1400 0000147f 
	7f2a1200 0000242a 08646200 00002313 56205000 00003649 07030000 00000008 
	22410000 0000001c 221c0000 00000041 7f1c2a00 00002a1c 3e080800 00000808 
	70300000 00000080 08080800 00000808 60600000 00000000 08040200 00002010 
	49453e00 00003e51 7f400000 00000042 49494600 00007249 494d3300 00002141 
	127f1000 00001814 45453900 00002745 49493100 00003c4a 11090700 00004121 
	49493600 00003649 49291e00 00004649 14000000 00000000 34000000 00000040 
	14224100 00000008 14141400 00001414 22140800 00000041 59090600 00000201 
	5d594e00 00003e41 11127c00 00007c12 49493600 00007f49 41412200 00003e41 
	41413e00 00007f41 49494100 00007f49 09090100 00007f09 41517300 00003e41 
	08087f00 00007f08 7f410000 00000041 413f0100 00002040 14224100 00007f08 
	40404000 00007f40 1c027f00 00007f02 08107f00 00007f04 41413e00 00003e41 
	09090600 00007f09 51215e00 00003e41 19294600 00007f09 49493200 00002649 
	7f010300 00000301 40403f00 00003f40 40201f00 00001f20 38403f00 00003f40 
	08146300 00006314 78040300 00000304 494d4300 00006159 41414100 0000007f 
	08102000 00000204 41417f00 00000041 01020400 00000402 40404000 00004040 
	07080000 00000003 54784000 00002054 44443800 00007f28 44442800 00003844 
	44287f00 00003844 54541800 00003854 7e090200 00000008 a49c7800 000018a4 
	04047800 00007f08 7d400000 00000044 403d0000 00002040 28440000 00007f10 
	7f400000 00000041 78047800 00007c04 04047800 00007c08 44443800 00003844 
	24241800 0000fc18 2418fc00 00001824 04040800 00007c08 54542400 00004854 
	3f442400 00000404 40207c00 00003c40 40201c00 00001c20 30403c00 00003c40 
	10284400 00004428 90907c00 00004c90 544c4400 00004464 36410000 00000008 
	77000000 00000000 36080000 00000041 02040200 00000201 23263c00 00003c26 
End CFunction
'
sub T32_test
  dim integer colours(7)=(red,blue,green,magenta,cyan,yellow,white,black)
  local integer x0, y0, x1, y1, wi, ht, colour=red
  local float tn
  local integer xvalues(3),yvalues(3)
' get these values for your display by using the calibrate program
  const xmin=125
  const xmax=1900
  const ymin=170
  const ymax=1950
  T32.setrot(0)
  wi= T32_params(0)
  ht= T32_params(1)
  T32.fillscreen(black)
  T32.drect(0, 0, wi, ht, red)
  T32.pstring(40, 40, "Welcome to the", 2, 0, green, black)
  T32.pstring(5, 70, "Micromite MK2", 3, 0, red, black)
  T32.pstring(60, 120, "in colour", 2, 0, yellow, black)
  T32.pstring(48, 170,  "SSD1289 controller", 1, 0, cyan, black)
  T32.pstring(54, 200,  "Touch Demo", 2, 0, green, black)
  pause 4000
  preparescreen
  do
    if T.getxy(xvalues(0),yvalues(0),1000) and T.getxy(xvalues(1),yvalues(1),1000) and T.getxy(xvalues(2),yvalues(2),1000) then 
      y0=filter(yvalues())
      x0=filter(xvalues())
      x1=(x0-xmin)*wi\(xmax-xmin)
      if y1<0 then y1=0
      y1=ht-((y0-ymin)*ht\(ymax-ymin))
      if x1<0 then x1=0
      if x1>=wi then x1=wi-1
'      print x0,y0,x1,y1
      if x1>=0 and y1>30 and x1<=wi-1 and y1<=ht-1 then
        T32.fcirc(x1,y1,3,colour)
      else
        if y1>ht-1 and x1<30 then
          preparescreen
        else
          if x1<wi-1 then colour=colours(x1\30)
        endif
      endif
    endif
  loop
end SUB
sub preparescreen
  T32.fillscreen(black)
  T32.FRECT(0,0,30,30,red)
  T32.FRECT(30,0,30,30,blue)
  T32.FRECT(60,0,30,30,green)
  T32.FRECT(90,0,30,30,magenta)
  T32.FRECT(120,0,30,30,cyan)
  T32.FRECT(150,0,30,30,yellow)
  T32.FRECT(180,0,30,30,white)
  T32.FRECT(210,0,30,30,black)
  T32.frect(0,30,240,1,white)
  T32.drect(0, 0, T32_params(T32_wi), T32_params(T32_ht), white)
end sub

function filter(values() as integer) as integer
  local integer AB,BC,AC
  AB=abs(values(0)-values(1))
  BC=ABS(values(1)-values(2))
  AC=ABS(values(0)-values(2))
  if AB<=BC and AB<=AC then filter=(values(0)+values(1))\2
  if BC<=AB and BC<=AC then filter=(values(1)+values(2))\2
  IF AC<=AB and AC<=BC then filter=(values(0)+values(2))\2
end function
